==> GMPhobos-0.2.9.tar: GMPhobos-0.2.9/Pod/Classes/Phobos.m <==
//
//  Phobos.m
//  GengmeiDoctor
//
//  Created by Thierry on 16/1/26.
//  Copyright © 2016年 wanmeizhensuo. All rights reserved.
//

#import "Phobos.h"
#import <GMCache/GMCache-umbrella.h>
#import <AdSupport/AdSupport.h>
#import "PhobosUtil.h"
#import "PhobosConfig.h"


static Phobos *sharedClient = nil;
static NSString *sdkVersion = @"110";

@interface Phobos ()

@property (strong, nonatomic) NSDateFormatter *dateFormatter;
@property (strong, nonatomic) NSString *appName;
@property (strong, nonatomic) NSString *channelId;
@property (strong, nonatomic) NSString *currentCityId;
@property (strong, nonatomic) NSString *appVersion;
@property (assign, nonatomic) NSInteger userId;
@property (assign, nonatomic) BOOL logEnabled;
@property (strong, nonatomic) NSString *businessId;
@property (strong, nonatomic) NSString *sessionId;

@property (strong, nonatomic) NSString *pageName;
@property (strong, nonatomic) NSString *refererPageName;
@property (strong, nonatomic) NSString *pageEnterTime;
@property (strong, nonatomic) NSString *pageExitTime;
@property (strong, nonatomic) NSMutableDictionary *pageEnterParam;

@end

@implementation Phobos

+ (Phobos *)clientWithAppName:(NSString *)appName channelId:(NSString *)channelId{
    return [[self alloc] initWithAppName:appName channelId:channelId];
}

+ (Phobos *)sharedClient{
    return sharedClient;
}

+ (void)setSharedClient:(Phobos *)client{
    sharedClient = client;
    [sharedClient handleEventAfterInit];
}

- (instancetype)initWithAppName:(NSString *)appName channelId:(NSString *)channelId{
    self = [super init];
    if (self) {
        _appName = appName;
        _channelId = channelId;
        _logEnabled = NO;
        _userId = 0;
        _currentCityId = @"";
        _appVersion = [self getAppVersion];
        [self setupNotification];
        [self handleSessionStart];
        
        phobosLog(@"starts to orbit");
    }
    return self;
}

- (void)setLogEnabled:(BOOL)value{
    _logEnabled = value;
}

- (void)setUserId:(NSInteger)userId{
    _userId = userId;
}

- (void)setCurrentCityId:(NSString *)currentCityId
{
    _currentCityId = currentCityId;
}

- (void)handleEventAfterInit{
    WMCacheService *cache = [WMCacheService sharedInstance];
    
    /** 每次打开APP埋点 **/
    [Phobos track:@"device_opened" attributes:@{} sendNow:YES];
    
    /** 第一次打开APP埋点 **/
    if (![cache fetchObjectAtDiskWithkey:PhobosHaveOpenApp]) {
        [Phobos track:@"device_activated" attributes:@{} sendNow:YES];
        [cache storeObjectAtDiskWithkey:PhobosHaveOpenApp object:PhobosHaveOpenApp];
    }
}


#pragma mark - notification handler
/**
 *  @brief 设置对APP的通知监听
 *
 *  @since 0.0.1
 */
- (void)setupNotification{
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(handleAppFinishLaunch:) name:UIApplicationDidFinishLaunchingNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(handleAppInForeground:) name:UIApplicationWillEnterForegroundNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(handleAppInBackgound:) name:UIApplicationDidEnterBackgroundNotification object:nil];
}

/**
 *  @brief 每次打开APP或返回前台，即Session开始的时候的处理
 *  @notification didFinishLaunch和willEnterForeground的时候都需要记录
 *
 *  @since 0.0.1
 */
- (void)handleSessionStart{
    _sessionId = [[NSUUID UUID] UUIDString];
    WMCacheService *cache = [WMCacheService sharedInstance];
    [cache storeObjectAtDiskWithkey:PhobosBeginTime object:[self currentTime]];
}

/**
 *  @brief 应用打开时的处理
 *
 *  @param sender
 *
 *  @since 0.0.1
 */
- (void)handleAppFinishLaunch:(id)sender{
    phobosLog(@"handleAppFinishLaunch");
    [self handleSessionStart];
}

/**
 *  @brief 应用进入前台的处理
 *
 *  @param sender
 *
 *  @since 0.0.1
 */
- (void)handleAppInForeground:(id)sender{
    phobosLog(@"handleAppInForeground");
    [self handleSessionStart];
    [self fetchDataAndSend];
}

/**
 *  @brief 应用进入后台的处理
 *
 *  @param sender
 *
 *  @since 0.0.1
 */
- (void)handleAppInBackgound:(id)sender{
    phobosLog(@"handleAppInBackgound");
    [self handleSessionOver];
    [self fetchDataAndSend];
}

/**
 *  @brief 会话结束时候的处理
 *
 *  @since 0.0.1
 */
- (void)handleSessionOver{
    WMCacheService *cache = [WMCacheService sharedInstance];
    //进入后台的同时，把记录时间同步到服务端，把已使用时间清空
    double beginTime = [[cache fetchObjectAtDiskWithkey:PhobosBeginTime] doubleValue];
    if (beginTime == 0) {
        return;
    }
    NSDate *date = [NSDate date];
    double endTime = [date timeIntervalSince1970];
    NSString *usedTime = [NSString stringWithFormat:@"%ld",(long)(endTime - beginTime)];
    NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithObjectsAndKeys:usedTime,@"duration",nil];
    [Phobos track:@"on_app_session_over" attributes:dict];
    [cache removeObjectAtDiskWithkey:PhobosBeginTime];
}


#pragma mark -  track event handler
+ (void)track:(NSString *)eventId{
    [Phobos track:eventId attributes:@{}];
}

+ (void)track:(NSString *)eventId attributes:(NSDictionary *)attributes{
    [self track:eventId attributes:attributes sendNow:NO];
    NSArray *array = [[WMCacheService sharedInstance] fetchObjectAtDiskWithkey:PhobosCacheKey];
    //超过一定数量的话，统一发送一次
    if (array.count > PhobosShardCount) {
        [sharedClient sendArray:array];
    }
}

+ (void)track:(NSString *)eventId attributes:(NSDictionary *)attributes sendNow:(BOOL)sendNow{
    NSDictionary *dict = [sharedClient prepareDictionaryForEvent:eventId attributes:attributes];
    if (sendNow) {
        NSArray *array = @[dict];
        [sharedClient sendArray:array];
    }else{
        [sharedClient save:dict];
    }
}

+ (void)trackJsEvent:(NSString *)jsonString{
    @try {
        NSData *data = [jsonString dataUsingEncoding:NSUnicodeStringEncoding];
        NSMutableDictionary *dict = [NSJSONSerialization JSONObjectWithData:data options:NSJSONReadingAllowFragments error:nil];
        
        id pa = dict[@"params"];
        NSDictionary *json;
        if ([pa isKindOfClass:[NSString class]]) {
            NSError *jsonError;
            NSData *objectData = [pa dataUsingEncoding:NSUTF8StringEncoding];
            json = [NSJSONSerialization JSONObjectWithData:objectData
                                                   options:NSJSONReadingMutableContainers
                                                     error:&jsonError];
            [Phobos track:dict[@"type"] attributes:json];
        }else{
            [Phobos track:dict[@"type"]];
        }
    }
    @catch (NSException *exception) {
        phobosLog(exception);
    }
}

#pragma mark - pv
- (void)onPageStart:(NSString *)pageName businessId:(NSString *)businessId referer:(NSString *)refererPageName{
    _pageEnterParam = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                       pageName?:@"",@"page_name",
                       self.businessId?:@"",@"business_id",
                       self.refererPageName?:@"",@"referer",
                       [self currentTime],@"in",nil];
}

- (void)onPageEnd{
    NSMutableDictionary *dict = [[NSMutableDictionary alloc] init];
    @try {
        [dict setObject:[self currentTime] forKey:@"out"];
        [dict setObject:@([_pageEnterParam[@"in"] doubleValue])?:@"" forKey:@"in"];
        [dict setObject:_pageEnterParam[@"page_name"]?:@"" forKey:@"page_name"];
        [dict setObject:_pageEnterParam[@"business_id"]?:@"" forKey:@"business_id"];
        [dict setObject:_pageEnterParam[@"referer"]?:@"" forKey:@"referer"];
        [dict setObject:@(0) forKey:@"fake"];
        
        [Phobos track:@"page_view" attributes:dict];
        [_pageEnterParam removeAllObjects];
    }
    @catch (NSException *exception) {
        phobosLog(exception);
    }
}

- (void)simulativePageViewEventWithPageName:(NSString *)pageName BusinessId:(NSString *)bid referer:(NSString *)referer{
    NSMutableDictionary *dict = [[NSMutableDictionary alloc] init];
    @try {
        [dict setObject:[self currentTime] forKey:@"in"];
        [dict setObject:[self currentTime] forKey:@"out"];
        [dict setObject:pageName?:@"" forKey:@"page_name"];
        [dict setObject:bid?:@"" forKey:@"business_id"];
        [dict setObject:referer?:@"" forKey:@"referer"];
        [dict setObject:@(1) forKey:@"fake"];
        
        [Phobos track:@"page_view" attributes:dict];
        [_pageEnterParam removeAllObjects];
    }
    @catch (NSException *exception) {
        phobosLog(exception);
    }
}

/**
 *  @brief 将埋点时间封装成词典数据
 *
 *  @return
 *
 *  @since 0.0.1
 */
- (NSDictionary *)prepareDictionaryForEvent:(NSString *)eventId attributes:(NSDictionary *)attributes{
    NSMutableDictionary *dict = [[NSMutableDictionary alloc] init];
    @try {
        NSString *currentTime = [self currentTime];
        NSMutableDictionary *deviceParams = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                                             [[[ASIdentifierManager sharedManager] advertisingIdentifier] UUIDString],@"device_id",
                                             @"ios",@"device_type",nil];
        NSMutableDictionary *appParams   = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                                            self.appName, @"name",
                                            self.appVersion, @"version",
                                            self.channelId,@"channel",
                                            self.currentCityId,@"current_city_id", nil];
        [dict setObject:eventId forKey:@"type"];
        [dict setObject:appParams forKey:@"app"];
        [dict setObject:sdkVersion forKey:@"version"];
        [dict setObject:deviceParams forKey:@"device"];
        [dict setObject:@(_userId) forKey:@"user_id"];
        [dict setObject:currentTime forKey:@"create_at"];
        [dict setObject:attributes forKey:@"params"];
        [dict setObject:_sessionId forKey:@"app_session_id"];
    }
    @catch (NSException *exception) {
        phobosLog(exception);
    }
    return dict;
}

/**
 *  @brief 保存数据到缓存层
 *
 *  @param data 数据
 *
 *  @since 0.0.1
 */
- (void)save:(NSDictionary *)data
{
    if (_logEnabled) {
        phobosLog([NSString stringWithFormat:@"save dictionary: %@",data]);
    }
    WMCacheService *cache = [WMCacheService sharedInstance];
    NSMutableArray *dataArray = [cache fetchObjectAtDiskWithkey:PhobosCacheKey];
    if (dataArray) {
        [dataArray addObject:data];
    }else{
        dataArray = [NSMutableArray arrayWithObject:data];
    }
    [cache storeObjectAtDiskWithkey:PhobosCacheKey object:dataArray];
}

/**
 *  @brief 从缓存中获取数据，并发送
 *
 *  @since 0.0.1
 */
- (void)fetchDataAndSend{
    NSArray *paramsArray = [[WMCacheService sharedInstance] fetchObjectAtDiskWithkey:PhobosCacheKey];
    if (paramsArray.count>0) {
        [self sendArray:paramsArray];
    }
}

/**
 *  @brief 发送数组
 *
 *  @param array
 *
 *  @since 0.0.1
 */
- (void)sendArray:(NSArray *)array {
    if (_logEnabled) {
        phobosLog([NSString stringWithFormat:@"array prepare to fly --✈: %@",array]);
    }
    @try {
        NSData *JSON = [self encodeJSON:array];
        NSData *compressedData = [PhobosUtil compressData:JSON];
        if (compressedData) {
            [PhobosUtil sendData:compressedData success:^(NSInteger code) {
                phobosLog(@"✈ ---------- ✈ data arrived Mars");
                [[WMCacheService sharedInstance] removeObjectAtDiskWithkey:PhobosCacheKey];
            }];
        }
    }
    @catch (NSException *exception) {
        phobosLog(exception);
    }
}

#pragma mark - helpers
/**
 *  @brief 将对象转成JSON格式数据
 *
 *  @param obj
 *
 *  @return
 *
 *  @since 0.0.1
 */
- (NSData *)encodeJSON:(id)obj {
    NSData *data = [NSJSONSerialization dataWithJSONObject:obj options:0 error:nil];
    return data;
}


/**
 *  @brief 获取当前时间的毫秒数
 *
 *  @return
 *
 *  @since 0.0.1
 */
- (NSString *)currentTime {
    NSDate *date = [NSDate date];
    NSTimeInterval interval = [date timeIntervalSince1970];
    NSString *timeIntervalStr = [NSString stringWithFormat:@"%ld",(long)interval];
    return timeIntervalStr;
}

/**
 *  @brief 获取当前APP的版本号
 *
 *  @return
 *
 *  @since 0.0.1
 */
- (NSString *)getAppVersion{
    NSString *buildVersion = [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleShortVersionString"];
    if (buildVersion) {
        return buildVersion;
    }else{
        return @"";
    }
}

- (BOOL)isNonEmpty:(NSString *)string {
    if (!string) {
        return NO;
    }
    NSMutableCharacterSet *emptyStringSet = [[NSMutableCharacterSet alloc] init];
    [emptyStringSet formUnionWithCharacterSet: [NSCharacterSet whitespaceAndNewlineCharacterSet]];
    [emptyStringSet formUnionWithCharacterSet: [NSCharacterSet characterSetWithCharactersInString: @"　"]];
    if ([string length] == 0) {
        return NO;
    }
    NSString* str = [string stringByTrimmingCharactersInSet:emptyStringSet];
    return [str length] > 0;
}

- (void)dealloc{
    if (self) {
        [[NSNotificationCenter defaultCenter] removeObserver:self];
    }
}


@end

==> GMPhobos-0.2.15.tar: GMPhobos-0.2.15/Pod/Classes/Phobos.m <==
//
//  Phobos.m
//  GengmeiDoctor
//
//  Created by Thierry on 16/1/26.
//  Copyright © 2016年 wanmeizhensuo. All rights reserved.
//

#import "Phobos.h"
#import <GMCache/GMCache-umbrella.h>
#import <AdSupport/AdSupport.h>
#import "PhobosUtil.h"
#import "PhobosConfig.h"
#import "UIViewController+Phobos.h"

static Phobos *sharedClient = nil;
static NSString *sdkVersion = @"110";

@interface Phobos ()

@property (strong, nonatomic) NSDateFormatter *dateFormatter;
@property (strong, nonatomic) NSString *appName;
@property (strong, nonatomic) NSString *channelId;
@property (strong, nonatomic) NSString *currentCityId;
@property (strong, nonatomic) NSString *appVersion;
@property (assign, nonatomic) NSInteger userId;
@property (assign, nonatomic) BOOL logEnabled;
@property (strong, nonatomic) NSString *sessionId;

@end

@implementation Phobos

+ (Phobos *)clientWithAppName:(NSString *)appName channelId:(NSString *)channelId{
    return [[self alloc] initWithAppName:appName channelId:channelId];
}

+ (Phobos *)sharedClient{
    return sharedClient;
}

+ (void)setSharedClient:(Phobos *)client{
    sharedClient = client;
    [sharedClient handleEventAfterInit];
}

- (instancetype)initWithAppName:(NSString *)appName channelId:(NSString *)channelId{
    self = [super init];
    if (self) {
        _appName = appName;
        _channelId = channelId;
        _logEnabled = NO;
        _userId = 0;
        _currentCityId = @"";
        _appVersion = [self getAppVersion];
        [self setupNotification];
        [self handleSessionStart];
        
        phobosLog(@"starts to orbit");
    }
    return self;
}

- (void)setLogEnabled:(BOOL)value{
    _logEnabled = value;
}

- (void)setUserId:(NSInteger)userId{
    _userId = userId;
}

- (void)setCurrentCityId:(NSString *)currentCityId
{
    _currentCityId = currentCityId;
}

- (void)handleEventAfterInit{
    WMCacheService *cache = [WMCacheService sharedInstance];
    
    /** 每次打开APP埋点 **/
    [Phobos track:@"device_opened" attributes:@{} sendNow:YES];
    
    /** 第一次打开APP埋点 **/
    if (![cache fetchObjectAtDiskWithkey:PhobosHaveOpenApp]) {
        [Phobos track:@"device_activated" attributes:@{} sendNow:YES];
        [Phobos track:@"device_activated" attributes:@{} sendNow:NO];
        [cache storeObjectAtDiskWithkey:PhobosHaveOpenApp object:PhobosHaveOpenApp];
    }
}

- (UINavigationController *)selectedNavigationController{
    if ([[UIApplication sharedApplication].keyWindow.rootViewController isKindOfClass:[UITabBarController class]]) {
        UITabBarController *tabbar = (UITabBarController *)[UIApplication sharedApplication].keyWindow.rootViewController;
        UINavigationController *navigationController = (UINavigationController *)tabbar.selectedViewController;
        return navigationController;
    }else{
        NSAssert(0, @"获取不到 selectedNavigationController %@", NSStringFromClass([UIApplication sharedApplication].keyWindow.rootViewController.class));
        return nil;
    }
}


#pragma mark - notification handler
/**
 *  @brief 设置对APP的通知监听
 *
 *  @since 0.0.1
 */
- (void)setupNotification{
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(handleAppFinishLaunch:) name:UIApplicationDidFinishLaunchingNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(handleAppInForeground:) name:UIApplicationWillEnterForegroundNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(handleAppInBackgound:) name:UIApplicationDidEnterBackgroundNotification object:nil];
}

/**
 *  @brief 每次打开APP或返回前台，即Session开始的时候的处理
 *  @notification didFinishLaunch和willEnterForeground的时候都需要记录
 *
 *  @since 0.0.1
 */
- (void)handleSessionStart{
    _sessionId = [[NSUUID UUID] UUIDString];
    WMCacheService *cache = [WMCacheService sharedInstance];
    [cache storeObjectAtDiskWithkey:PhobosBeginTime object:[self currentTime]];
}

/**
 *  @brief 应用打开时的处理
 *
 *  @param sender
 *
 *  @since 0.0.1
 */
- (void)handleAppFinishLaunch:(id)sender{
    phobosLog(@"handleAppFinishLaunch");
    [self handleSessionStart];
}

/**
 *  @brief 应用进入前台的处理
 *
 *  @param sender
 *
 *  @since 0.0.1
 */
- (void)handleAppInForeground:(id)sender{
    phobosLog(@"handleAppInForeground");
    [self handleSessionStart];
    [self fetchDataAndSend];
    [self handlePVEventAppInForeground];
}

/**
 *  @brief 应用进入后台的处理
 *
 *  @param sender
 *
 *  @since 0.0.1
 */
- (void)handleAppInBackgound:(id)sender{
    phobosLog(@"handleAppInBackgound");
    [self handleSessionOver];
    [self fetchDataAndSend];
    [self handlePVEventAppInBackgound];
}

/**
 *  @brief 会话结束时候的处理
 *
 *  @since 0.0.1
 */
- (void)handleSessionOver{
    WMCacheService *cache = [WMCacheService sharedInstance];
    //进入后台的同时，把记录时间同步到服务端，把已使用时间清空
    double beginTime = [[cache fetchObjectAtDiskWithkey:PhobosBeginTime] doubleValue];
    if (beginTime == 0) {
        return;
    }
    NSDate *date = [NSDate date];
    double endTime = [date timeIntervalSince1970];
    NSString *usedTime = [NSString stringWithFormat:@"%ld",(long)(endTime - beginTime)];
    NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithObjectsAndKeys:usedTime,@"duration",nil];
    [Phobos track:@"on_app_session_over" attributes:dict];
    [cache removeObjectAtDiskWithkey:PhobosBeginTime];
}

/**
 APP从后台到前台的时候，重新初始化pagename等信息
 @author zhaiguojun 16-10-11 in (null)
 */
- (void)handlePVEventAppInForeground {
    NSString *pageName = self.selectedNavigationController.visibleViewController.pageName;
    NSString *businessId = self.selectedNavigationController.visibleViewController.businessId;
    NSString *regerer = self.selectedNavigationController.visibleViewController.referer;
    self.selectedNavigationController.visibleViewController.inTime = [self currentTime];
    [self onPageStart:pageName businessId:businessId referer:regerer inTime:[self currentTime]];
}

/**
 APP进到后台的时候，把当前pageview时间结束
 @author zhaiguojun 16-10-11 in (null)
 */
- (void)handlePVEventAppInBackgound {
    NSString *pageName = self.selectedNavigationController.visibleViewController.pageName;
    NSString *businessId = self.selectedNavigationController.visibleViewController.businessId;
    NSString *referer = self.selectedNavigationController.visibleViewController.referer;
    NSString *inTime = self.selectedNavigationController.visibleViewController.inTime;
    
    [self onPageEnd:pageName businessId:businessId referer:referer inTime:inTime];
}

#pragma mark -  track event handler
+ (void)track:(NSString *)eventId{
    [Phobos track:eventId attributes:@{}];
}

+ (void)track:(NSString *)eventId attributes:(NSDictionary *)attributes{
    [self track:eventId attributes:attributes sendNow:NO];
    NSArray *array = [[WMCacheService sharedInstance] fetchObjectAtDiskWithkey:PhobosCacheKey];
    //超过一定数量的话，统一发送一次
    if (array.count > PhobosShardCount) {
        [sharedClient sendArray:array cleanCacheRightNow:YES];
    }
}

+ (void)track:(NSString *)eventId attributes:(NSDictionary *)attributes sendNow:(BOOL)sendNow{
    NSDictionary *dict = [sharedClient prepareDictionaryForEvent:eventId attributes:attributes];
    if (sendNow) {
        NSArray *array = @[dict];
        // 实时发送的埋点，不能立即清楚缓存
        [sharedClient sendArray:array cleanCacheRightNow:NO];
    }else{
        [sharedClient save:dict];
    }
}

+ (void)trackJsEvent:(NSString *)jsonString{
    @try {
        NSData *data = [jsonString dataUsingEncoding:NSUnicodeStringEncoding];
        NSMutableDictionary *dict = [NSJSONSerialization JSONObjectWithData:data options:NSJSONReadingAllowFragments error:nil];
        
        id pa = dict[@"params"];
        NSDictionary *json;
        if ([pa isKindOfClass:[NSString class]]) {
            NSError *jsonError;
            NSData *objectData = [pa dataUsingEncoding:NSUTF8StringEncoding];
            json = [NSJSONSerialization JSONObjectWithData:objectData
                                                   options:NSJSONReadingMutableContainers
                                                     error:&jsonError];
            [Phobos track:dict[@"type"] attributes:json];
        }else{
            [Phobos track:dict[@"type"]];
        }
    }
    @catch (NSException *exception) {
        phobosLog(exception);
    }
}

#pragma mark - pv
- (void)onPageStart:(NSString *)pageName businessId:(NSString *)businessId referer:(NSString *)refererPageName inTime:(NSString *)time{
    //目前来说，变成了空方法，其主要功能是给当前VC的属性赋值，下个版本优化，挪到UIViewController+Phobos里去
}

- (void)onPageEnd:(NSString *)pageName businessId:(NSString *)businessId referer:(NSString *)refererPageName inTime:(NSString *)time{
    if (![self isNonEmpty:pageName]) {
        return;
    }
    NSMutableDictionary *dict = [[NSMutableDictionary alloc] init];
    @try {
        
        [dict setObject:[self currentTime] forKey:@"out"];
        [dict setObject:time?:@"" forKey:@"in"];
        [dict setObject:pageName forKey:@"page_name"];
        [dict setObject:businessId?:@"" forKey:@"business_id"];
        [dict setObject:refererPageName?:@"" forKey:@"referer"];
        [dict setObject:@(0) forKey:@"fake"];
        
        [Phobos track:@"page_view" attributes:dict];
    }
    @catch (NSException *exception) {
        phobosLog(exception);
    }
}

- (void)simulativePageViewEventWithPageName:(NSString *)pageName BusinessId:(NSString *)bid referer:(NSString *)referer{
    NSMutableDictionary *dict = [[NSMutableDictionary alloc] init];
    @try {
        [dict setObject:[self currentTime] forKey:@"in"];
        [dict setObject:[self currentTime] forKey:@"out"];
        [dict setObject:pageName?:@"" forKey:@"page_name"];
        [dict setObject:bid?:@"" forKey:@"business_id"];
        [dict setObject:referer?:@"" forKey:@"referer"];
        [dict setObject:@(1) forKey:@"fake"];
        
        [Phobos track:@"page_view" attributes:dict];
    }
    @catch (NSException *exception) {
        phobosLog(exception);
    }
}

/**
 *  @brief 将埋点时间封装成词典数据
 *
 *  @return
 *
 *  @since 0.0.1
 */
- (NSDictionary *)prepareDictionaryForEvent:(NSString *)eventId attributes:(NSDictionary *)attributes{
    NSMutableDictionary *dict = [[NSMutableDictionary alloc] init];
    @try {
        NSString *currentTime = [self currentTime];
        NSMutableDictionary *deviceParams = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                                             [[[ASIdentifierManager sharedManager] advertisingIdentifier] UUIDString],@"device_id",
                                             [[[UIDevice currentDevice] identifierForVendor] UUIDString],@"idfv",
                                             @"ios",@"device_type",nil];
        NSMutableDictionary *appParams   = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                                            self.appName, @"name",
                                            self.appVersion, @"version",
                                            self.channelId,@"channel",
                                            self.currentCityId,@"current_city_id", nil];
        [dict setObject:eventId forKey:@"type"];
        [dict setObject:appParams forKey:@"app"];
        [dict setObject:sdkVersion forKey:@"version"];
        [dict setObject:deviceParams forKey:@"device"];
        [dict setObject:@(_userId) forKey:@"user_id"];
        [dict setObject:currentTime forKey:@"create_at"];
        [dict setObject:attributes forKey:@"params"];
        [dict setObject:_sessionId forKey:@"app_session_id"];
    }
    @catch (NSException *exception) {
        phobosLog(exception);
    }
    return dict;
}

/**
 *  @brief 保存数据到缓存层
 *
 *  @param data 数据
 *
 *  @since 0.0.1
 */
- (void)save:(NSDictionary *)data
{
    if (_logEnabled) {
        phobosLog([NSString stringWithFormat:@"save dictionary: %@",data]);
    }
    WMCacheService *cache = [WMCacheService sharedInstance];
    NSMutableArray *dataArray = [cache fetchObjectAtDiskWithkey:PhobosCacheKey];
    if (dataArray) {
        [dataArray addObject:data];
    }else{
        dataArray = [NSMutableArray arrayWithObject:data];
    }
    [cache storeObjectAtDiskWithkey:PhobosCacheKey object:dataArray];
}

/**
 *  @brief 从缓存中获取数据，并发送
 *
 *  @since 0.0.1
 */
- (void)fetchDataAndSend{
    NSArray *paramsArray = [[WMCacheService sharedInstance] fetchObjectAtDiskWithkey:PhobosCacheKey];
    if (paramsArray.count>0) {
        [self sendArray:paramsArray cleanCacheRightNow:YES];
    }
}

/**
 向Mars发送埋点数据，如果是实时发送的，http请求成功之后，不清楚已有的缓存数据。
 针对普通埋点数据，发送成功之后，立即删除本地的缓存。
 @author zhaiguojun 16-10-17 in (null)
 @param array 参数
 @param now   是否立即清楚缓存
 */
- (void)sendArray:(NSArray *)array cleanCacheRightNow:(BOOL)clean {
    if (_logEnabled) {
        phobosLog([NSString stringWithFormat:@"array prepare to fly --✈: %@",array]);
    }
    @try {
        NSData *JSON = [self encodeJSON:array];
        NSData *compressedData = [PhobosUtil compressData:JSON];
        if (compressedData) {
            [PhobosUtil sendData:compressedData success:^(NSInteger code) {
                phobosLog(@"✈ ---------- ✈ data arrived Mars");
                if (clean) {
                    [[WMCacheService sharedInstance] removeObjectAtDiskWithkey:PhobosCacheKey];
                }
            }];
        }
    }
    @catch (NSException *exception) {
        phobosLog(exception);
    }
}

#pragma mark - helpers
/**
 *  @brief 将对象转成JSON格式数据
 *
 *  @param obj
 *
 *  @return
 *
 *  @since 0.0.1
 */
- (NSData *)encodeJSON:(id)obj {
    NSData *data = [NSJSONSerialization dataWithJSONObject:obj options:0 error:nil];
    return data;
}


/**
 *  @brief 获取当前时间的毫秒数
 *
 *  @return
 *
 *  @since 0.0.1
 */
- (NSString *)currentTime {
    NSDate *date = [NSDate date];
    NSTimeInterval interval = [date timeIntervalSince1970];
    NSString *timeIntervalStr = [NSString stringWithFormat:@"%ld",(long)interval];
    return timeIntervalStr;
}

/**
 *  @brief 获取当前APP的版本号
 *
 *  @return
 *
 *  @since 0.0.1
 */
- (NSString *)getAppVersion{
    NSString *buildVersion = [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleShortVersionString"];
    if (buildVersion) {
        return buildVersion;
    }else{
        return @"";
    }
}

- (BOOL)isNonEmpty:(NSString *)string {
    if (!string) {
        return NO;
    }
    NSMutableCharacterSet *emptyStringSet = [[NSMutableCharacterSet alloc] init];
    [emptyStringSet formUnionWithCharacterSet: [NSCharacterSet whitespaceAndNewlineCharacterSet]];
    [emptyStringSet formUnionWithCharacterSet: [NSCharacterSet characterSetWithCharactersInString: @"　"]];
    if ([string length] == 0) {
        return NO;
    }
    NSString* str = [string stringByTrimmingCharactersInSet:emptyStringSet];
    return [str length] > 0;
}

- (void)dealloc{
    if (self) {
        [[NSNotificationCenter defaultCenter] removeObserver:self];
    }
}

@end
